/*
 File    : util/p/rtb_createsonarproperties.p
 Module  : rtb/util
 Author  : jeff
 Created : 12/29/22
 About   : Initially created under Task 4649.
 Purpose : Build a Sonar Project File
 
 Parameters:
   INPUT
     pcWorkspace - Roundtable Workspace to scan
     pcToken     - Sonar user token
   OUTPUT
     pcError     - Error message
     
  Usage Example:
  
  {adecomm/appserv.i}
  {rtb/i/rtb_appserv.i}

  RUN rtb/util/p/rtb_createsonarproperties.p ON rtbGetServiceHandle()
    ("devel", "MyToken", OUTPUT cError).
*/
USING rtb.util.*.
ROUTINE-LEVEL ON ERROR UNDO, THROW.

DEFINE INPUT  PARAMETER pcWorkspace AS CHARACTER NO-UNDO.
DEFINE INPUT  PARAMETER pcToken     AS CHARACTER NO-UNDO.
DEFINE OUTPUT PARAMETER pcError     AS CHARACTER NO-UNDO.

{rtb/i/rtb_objtypes.i}

&SCOPED-DEFINE DF-FOLDER "dump"
&SCOPED-DEFINE SOURCE-CODEPAGE "ISO8859-1"

/*
 The default assumption is to scan from the Workspace root and that r-code files are 
 in the same directories as the source files. Modify as needed.
   
 For example:

   &SCOPED-DEFINE SONAR-SOURCES "src"
   &SCOPED-DEFINE SONAR-OE-BINARIES "bin"
*/
&SCOPED-DEFINE SONAR-SOURCES "."
&SCOPED-DEFINE SONAR-OE-BINARIES "."

/*
 By default, the Roundtable 'list' directories are excluded. If you want to exclude
 additional directories, add them to the preprocessor definition below.
 
 For example:
   &SCOPED-DEFINE SONAR-EXCLUSIONS "dump/,images/"
*/
&SCOPED-DEFINE SONAR-EXCLUSIONS ""
                                               
DEFINE VARIABLE sonarProperties AS SonarProperties NO-UNDO.

DEFINE TEMP-TABLE rtbWorkspace
  {rtb/sdo/rtb_u_ws.i}
  {rtb/admtwo/robjflds.i}.
  
DEFINE TEMP-TABLE rtbObject
  {rtb/sdo/rtb_u_obj.i}
  {rtb/admtwo/robjflds.i}.

DEFINE TEMP-TABLE rtbDbalias
  {rtb/sdo/rtb_u_dbalias.i}
  {rtb/admtwo/robjflds.i}.

DEFINE TEMP-TABLE rtbRelease
  {rtb/sdo/rtb_u_rel.i}
  {rtb/admtwo/robjflds.i}.
  
DEFINE TEMP-TABLE rtbSubtype
  {rtb/sdo/rtb_u_subt.i}
  {rtb/admtwo/robjflds.i}.
  
RUN build_properties_file.
  
RETURN.

CATCH e AS Progress.Lang.SysError:
  pcError = DYNAMIC-FUNCTION('fnRtbErrorHandlerObject', e).    
END CATCH.

CATCH a AS Progress.Lang.AppError :
  pcError = DYNAMIC-FUNCTION('fnRtbErrorHandlerObject', a).     
END CATCH.

PROCEDURE build_properties_file PRIVATE:

  RUN rtb/proxy/p/rtbGetWorkspace.p (pcWorkspace, OUTPUT TABLE rtbWorkspace).

  FIND FIRST rtbWorkspace.

  sonarProperties = NEW SonarProperties().

  ASSIGN 
    sonarProperties:sonar-projectKey         = "rtb:" + rtbWorkspace.wspace-id
    sonarProperties:sonar-projectName        = rtbWorkspace.wspace-id
    sonarProperties:sonar-projectDescription = rtbWorkspace.description
    sonarProperties:sonar-sources            = {&SONAR-SOURCES}
    sonarProperties:sonar-oe-binaries        = {&SONAR-OE-BINARIES}
    sonarProperties:sonar-sourceEncoding     = {&SOURCE-CODEPAGE}
    sonarProperties:sonar-oe-dlc             = DYNAMIC-FUNCTION('fnRtbPath', SEARCH("progress.cfg")) /* OS-GETENV('DLC') may not be correct */
    sonarProperties:sonar-login              = pcToken.

  IF {&SONAR-EXCLUSIONS} <> "" THEN 
    sonarProperties:sonar-exclusions = sonarProperties:sonar-exclusions + "," + {&SONAR-EXCLUSIONS}.

  RUN set_version.
  RUN set_propath.
  RUN set_databases.
  
  /*
   Uncomment the line below if you use more than the standard OE extensions for your source code.
  */  

  /*RUN set_suffixes.*/ 
  
  sonarProperties:writeFile(DYNAMIC-FUNCTION('fnRtbGetWorkspaceRoot', rtbWorkspace.wspace-id) + "/sonar-project.properties").
  
  RETURN.
  
  FINALLY:
    DELETE OBJECT sonarProperties NO-ERROR.
  END FINALLY.
  
END PROCEDURE.

PROCEDURE set_propath PRIVATE:
  /*
   Build propath value based on each entry in Workspace path
  */
  DEFINE VARIABLE cEntry   AS CHARACTER  NO-UNDO.
  DEFINE VARIABLE cWsRoot  AS CHARACTER  NO-UNDO.
  DEFINE VARIABLE cPropath AS CHARACTER  NO-UNDO.
  DEFINE VARIABLE i        AS INTEGER    NO-UNDO.

  DO i = 1 TO NUM-ENTRIES(rtbWorkspace.wspace-path):
    
    cEntry = ENTRY(i,rtbWorkspace.wspace-path).

    IF i = 1 THEN DO: 
      ASSIGN 
        cPropath = "."
        cWsRoot  = cEntry.
      NEXT.
    END.
    
    cPropath = cPropath + "," + SUBSTRING(cEntry, LENGTH(cWsRoot) + 2).
    
  END.
  
  sonarProperties:sonar-oe-propath = cPropath.
  
  RETURN.
    
END PROCEDURE.

PROCEDURE set_databases PRIVATE:
  /*
   Fetch each Workspace PDBASE, build db and alias lists, and dump DFs 
  */
  DEFINE VARIABLE cAliasList AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cDbList    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cRowident  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE hDbConn    AS HANDLE    NO-UNDO.
  DEFINE VARIABLE hBuf       AS HANDLE    NO-UNDO.
  DEFINE VARIABLE lOk        AS LOGICAL   NO-UNDO.
  
  RUN rtb/proxy/p/rtbGetWorkspaceObjects.p (INPUT-OUTPUT cRowident, 0, pcWorkspace, {&PDBASE}, "*", OUTPUT TABLE rtbObject).
  
  RUN rtb/proxy/p/rtbGetWorkspaceDbConnections.p (pcWorkspace, OUTPUT TABLE-HANDLE hDbConn).
  
  hBuf = hDbConn:DEFAULT-BUFFER-HANDLE.
  
  FOR EACH rtbObject:
    
    lOk = hBuf:FIND-UNIQUE("where object = " + QUOTER(rtbObject.object)) NO-ERROR. 
    
    IF NOT(lOk) THEN NEXT.  /* skip any pdbase objects with no connect entries */
      
    cDbList = cDblist + (IF cDbList = "" THEN "" ELSE ",") + {&DF-FOLDER} + "/" + rtbObject.object + ".df".
    
    cRowident = "".
    
    RUN rtb/proxy/p/rtbGetVersionDbAliases.p (INPUT-OUTPUT cRowident, 0, rtbObject.object, rtbObject.pmod, rtbObject.version, OUTPUT TABLE rtbDbalias).
            
    FOR EACH rtbDbalias BREAK BY rtbDbalias.OBJECT:
      
      IF FIRST-OF(rtbDbalias.OBJECT) THEN
        cAliasList = cAliasList + (IF cAliasList = "" THEN "" ELSE ";") + rtbObject.object.
      
      cAliasList = cAliasList + "," + rtbDbalias.alias-name.
      
    END.    
    
    EMPTY TEMP-TABLE rtbDbalias.
    
    RUN dump_df (hBuf).
    
  END.

  ASSIGN 
    sonarProperties:sonar-oe-databases = cDbList
    sonarProperties:sonar-oe-aliases   = cAliasList.
  
  RETURN.
  
  FINALLY:
    DELETE OBJECT hDbConn NO-ERROR.
  END FINALLY.  
  
END PROCEDURE.

PROCEDURE set_version PRIVATE:
  /*
   Set the project version equal to the latest Workspace release
  */
  DEFINE VARIABLE cRowident AS CHARACTER NO-UNDO.
  
  RUN rtb/proxy/p/rtbGetWorkspaceReleases.p (INPUT-OUTPUT cRowident, 1, pcWorkspace, OUTPUT TABLE rtbRelease).
  
  FIND FIRST rtbRelease.

  sonarProperties:sonar-projectVersion = STRING(rtbRelease.release-num).
  
  RETURN.
  
END PROCEDURE.

PROCEDURE set_suffixes PRIVATE:
  /*
   Set the suffix list based on Subtype extensions. 
  */
  DEFINE VARIABLE cRowident AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cExtList  AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cExt      AS CHARACTER NO-UNDO.
  DEFINE VARIABLE i         AS INTEGER   NO-UNDO.
  
  RUN rtb/proxy/p/rtbGetSubtypes.p (INPUT-OUTPUT cRowident, 0, FALSE, OUTPUT TABLE rtbSubtype).
  
  FOR EACH rtbSubtype:
    
    IF rtbSubtype.binary THEN NEXT.
      
    DO i = 1 TO NUM-ENTRIES(rtbSubtype.subtPart-exts):

      cExt = ENTRY(i, rtbSubtype.subtPart-exts).
      
      IF cExt = "" THEN NEXT.

      IF LOOKUP(cExt, cExtList) > 0 THEN NEXT.
                  
      cExtList = cExtList + (IF cExtList = "" THEN "" ELSE ",") + cExt.
        
    END.
        
  END.

  sonarProperties:sonar-oe-file-suffixes = cExtList.
  
  RETURN.
  
END PROCEDURE.

PROCEDURE dump_df PRIVATE:
  /*
   Dump the database contained in the given buffer using the standard OE utilities. 
  */
  DEFINE INPUT PARAMETER phBuf AS HANDLE NO-UNDO.
  
  DEFINE VARIABLE cDictdb AS CHARACTER NO-UNDO.
  DEFINE VARIABLE cTarget AS CHARACTER NO-UNDO.
  DEFINE VARIABLE h       AS HANDLE    NO-UNDO.

  cDictdb = LDBNAME("DICTDB").

  IF NOT CONNECTED(phBuf::object) THEN 
    CONNECT VALUE(phBuf::connect-params).

  CREATE ALIAS DICTDB FOR DATABASE VALUE(phBuf::object).

  cTarget = DYNAMIC-FUNCTION('fnRtbGetWorkspaceRoot', rtbWorkspace.wspace-id) + "/" + {&DF-FOLDER} + "/" + phBuf::object + ".df".
  
  DYNAMIC-FUNCTION('fnRtbMakeOSPath', cTarget).
  
  RUN prodict/dump_df.p PERSISTENT SET h ("all", cTarget, ?).
  RUN setSilent IN h (TRUE).
  RUN doDump IN h.

  DELETE PROCEDURE h.

  DELETE ALIAS dictdb.
  CREATE ALIAS dictdb FOR DATABASE VALUE(cDictdb) NO-ERROR.

  DISCONNECT VALUE(phBuf::object).
  
  RETURN.

END PROCEDURE.
